@@ -1,8 +1,8 @@
 /*
  * Mesa 3-D graphics library
- * Version:  6.3
+ * Version:  6.5
  *
- * Copyright (C) 1999-2005  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2006  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -38,6 +38,7 @@
 
 #include "s_atifragshader.h"
 #include "s_alpha.h"
+#include "s_arbshader.h"
 #include "s_blend.h"
 #include "s_context.h"
 #include "s_depth.h"
@@ -47,7 +48,7 @@
 #include "s_nvfragprog.h"
 #include "s_span.h"
 #include "s_stencil.h"
-#include "s_texture.h"
+#include "s_texcombine.h"
 
 
 /**
@@ -58,7 +59,7 @@ void
 _swrast_span_default_z( GLcontext *ctx, struct sw_span *span )
 {
    const GLfloat depthMax = ctx->DrawBuffer->_DepthMaxF;
-   if (ctx->Visual.depthBits <= 16)
+   if (ctx->DrawBuffer->Visual.depthBits <= 16)
       span->z = FloatToFixed(ctx->Current.RasterPos[2] * depthMax + 0.5F);
    else
       span->z = (GLint) (ctx->Current.RasterPos[2] * depthMax + 0.5F);
@@ -126,7 +127,7 @@ void
 _swrast_span_default_texcoords( GLcontext *ctx, struct sw_span *span )
 {
    GLuint i;
-   for (i = 0; i < ctx->Const.MaxTextureUnits; i++) {
+   for (i = 0; i < ctx->Const.MaxTextureCoordUnits; i++) {
       const GLfloat *tc = ctx->Current.RasterTexCoords[i];
       if (ctx->FragmentProgram._Active || ctx->ATIFragmentShader._Enabled) {
          COPY_4V(span->tex[i], tc);
@@ -311,9 +312,9 @@ _swrast_span_interpolate_z( const GLcontext *ctx, struct sw_span *span )
    ASSERT((span->interpMask & SPAN_Z)  &&
 	  !(span->arrayMask & SPAN_Z));
 
-   if (ctx->Visual.depthBits <= 16) {
+   if (ctx->DrawBuffer->Visual.depthBits <= 16) {
       GLfixed zval = span->z;
-      GLdepth *z = span->array->z; 
+      GLuint *z = span->array->z; 
       for (i = 0; i < n; i++) {
          z[i] = FixedToInt(zval);
          zval += span->zStep;
@@ -321,8 +322,8 @@ _swrast_span_interpolate_z( const GLcontext *ctx, struct sw_span *span )
    }
    else {
       /* Deep Z buffer, no fixed->int shift */
-      GLfixed zval = span->z;
-      GLdepth *z = span->array->z;
+      GLuint zval = span->z;
+      GLuint *z = span->array->z;
       for (i = 0; i < n; i++) {
          z[i] = zval;
          zval += span->zStep;
@@ -400,6 +401,7 @@ interpolate_texcoords(GLcontext *ctx, struct sw_span *span)
       /* multitexture */
       GLuint u;
       span->arrayMask |= SPAN_TEXTURE;
+      /* XXX CoordUnits vs. ImageUnits */
       for (u = 0; u < ctx->Const.MaxTextureUnits; u++) {
          if (ctx->Texture._EnabledCoordUnits & (1 << u)) {
             const struct gl_texture_object *obj =ctx->Texture.Unit[u]._Current;
@@ -433,7 +435,8 @@ interpolate_texcoords(GLcontext *ctx, struct sw_span *span)
                GLfloat r = span->tex[u][2];
                GLfloat q = span->tex[u][3];
                GLuint i;
-               if (ctx->FragmentProgram._Active) {
+               if (ctx->FragmentProgram._Active || ctx->ATIFragmentShader._Enabled ||
+                   ctx->ShaderObjects._FragmentShaderPresent) {
                   /* do perspective correction but don't divide s, t, r by q */
                   const GLfloat dwdx = span->dwdx;
                   GLfloat w = span->w;
@@ -484,7 +487,8 @@ interpolate_texcoords(GLcontext *ctx, struct sw_span *span)
                GLfloat r = span->tex[u][2];
                GLfloat q = span->tex[u][3];
                GLuint i;
-               if (ctx->FragmentProgram._Active) {
+               if (ctx->FragmentProgram._Active || ctx->ATIFragmentShader._Enabled ||
+                   ctx->ShaderObjects._FragmentShaderPresent) {
                   /* do perspective correction but don't divide s, t, r by q */
                   const GLfloat dwdx = span->dwdx;
                   GLfloat w = span->w;
@@ -567,7 +571,8 @@ interpolate_texcoords(GLcontext *ctx, struct sw_span *span)
          GLfloat r = span->tex[0][2];
          GLfloat q = span->tex[0][3];
          GLuint i;
-         if (ctx->FragmentProgram._Active) {
+         if (ctx->FragmentProgram._Active || ctx->ATIFragmentShader._Enabled ||
+             ctx->ShaderObjects._FragmentShaderPresent) {
             /* do perspective correction but don't divide s, t, r by q */
             const GLfloat dwdx = span->dwdx;
             GLfloat w = span->w;
@@ -618,7 +623,8 @@ interpolate_texcoords(GLcontext *ctx, struct sw_span *span)
          GLfloat r = span->tex[0][2];
          GLfloat q = span->tex[0][3];
          GLuint i;
-         if (ctx->FragmentProgram._Active) {
+         if (ctx->FragmentProgram._Active || ctx->ATIFragmentShader._Enabled ||
+             ctx->ShaderObjects._FragmentShaderPresent) {
             /* do perspective correction but don't divide s, t, r by q */
             const GLfloat dwdx = span->dwdx;
             GLfloat w = span->w;
@@ -666,6 +672,38 @@ interpolate_texcoords(GLcontext *ctx, struct sw_span *span)
 }
 
 
+/**
+ * Fill in the span.varying array from the interpolation values.
+ */
+static void
+interpolate_varying(GLcontext *ctx, struct sw_span *span)
+{
+   GLuint i, j;
+
+   ASSERT(span->interpMask & SPAN_VARYING);
+   ASSERT(!(span->arrayMask & SPAN_VARYING));
+
+   span->arrayMask |= SPAN_VARYING;
+
+   for (i = 0; i < MAX_VARYING_VECTORS; i++) {
+      for (j = 0; j < VARYINGS_PER_VECTOR; j++) {
+         const GLfloat dvdx = span->varStepX[i][j];
+         GLfloat v = span->var[i][j];
+         const GLfloat dwdx = span->dwdx;
+         GLfloat w = span->w;
+         GLuint k;
+
+         for (k = 0; k < span->end; k++) {
+            GLfloat invW = 1.0f / w;
+            span->array->varying[k][i][j] = v * invW;
+            v += dvdx;
+            w += dwdx;
+         }
+      }
+   }
+}
+
+
 /**
  * Apply the current polygon stipple pattern to a span of pixels.
  */
@@ -776,8 +814,8 @@ _swrast_write_index_span( GLcontext *ctx, struct sw_span *span)
    const SWcontext *swrast = SWRAST_CONTEXT(ctx);
    const struct gl_framebuffer *fb = ctx->DrawBuffer;
    const GLuint output = 0;
-   const GLuint origInterpMask = span->interpMask;
-   const GLuint origArrayMask = span->arrayMask;
+   const GLbitfield origInterpMask = span->interpMask;
+   const GLbitfield origArrayMask = span->arrayMask;
    GLuint buf;
 
    ASSERT(span->end <= MAX_WIDTH);
@@ -803,7 +841,7 @@ _swrast_write_index_span( GLcontext *ctx, struct sw_span *span)
    }
 
    /* Depth bounds test */
-   if (ctx->Depth.BoundsTest && ctx->Visual.depthBits > 0) {
+   if (ctx->Depth.BoundsTest && ctx->DrawBuffer->Visual.depthBits > 0) {
       if (!_swrast_depth_bounds_test(ctx, span)) {
          return;
       }
@@ -850,17 +888,13 @@ _swrast_write_index_span( GLcontext *ctx, struct sw_span *span)
       }
    }
 
-   /* if we get here, something passed the depth test */
-   if (ctx->Depth.OcclusionTest) {
-      ctx->OcclusionResult = GL_TRUE;
-   }
-
 #if FEATURE_ARB_occlusion_query
-   if (ctx->Occlusion.Active) {
+   if (ctx->Query.CurrentOcclusionObject) {
       /* update count of 'passed' fragments */
+      struct gl_query_object *q = ctx->Query.CurrentOcclusionObject;
       GLuint i;
       for (i = 0; i < span->end; i++)
-         ctx->Occlusion.PassedCounter += span->array->mask[i];
+         q->Result += span->array->mask[i];
    }
 #endif
 
@@ -872,7 +906,7 @@ _swrast_write_index_span( GLcontext *ctx, struct sw_span *span)
    }
 
    /* Interpolate the color indexes if needed */
-   if (ctx->Fog.Enabled ||
+   if (swrast->_FogEnabled ||
        ctx->Color.IndexLogicOpEnabled ||
        ctx->Color.IndexMask != 0xffffffff ||
        (span->arrayMask & SPAN_COVERAGE)) {
@@ -882,7 +916,7 @@ _swrast_write_index_span( GLcontext *ctx, struct sw_span *span)
    }
 
    /* Fog */
-   if (ctx->Fog.Enabled) {
+   if (swrast->_FogEnabled) {
       _swrast_fog_ci_span(ctx, span);
    }
 
@@ -988,11 +1022,6 @@ _swrast_write_index_span( GLcontext *ctx, struct sw_span *span)
       }
    }
 
-#if OLD_RENDERBUFFER
-   /* restore default dest buffer */
-   _swrast_use_draw_buffer(ctx);
-#endif
-
    span->interpMask = origInterpMask;
    span->arrayMask = origArrayMask;
 }
@@ -1034,9 +1063,6 @@ add_colors(GLuint n, GLchan rgba[][4], GLchan specular[][4] )
 static void
 multi_write_rgba_span( GLcontext *ctx, struct sw_span *span )
 {
-#if OLD_RENDERBUFFER
-   SWcontext *swrast = SWRAST_CONTEXT(ctx);
-#endif
    const GLuint colorMask = *((GLuint *) ctx->Color.ColorMask);
    struct gl_framebuffer *fb = ctx->DrawBuffer;
    const GLuint output = 0;
@@ -1049,15 +1075,6 @@ multi_write_rgba_span( GLcontext *ctx, struct sw_span *span )
       struct gl_renderbuffer *rb = fb->_ColorDrawBuffers[output][i];
       GLchan rgbaTmp[MAX_WIDTH][4];
 
-#if OLD_RENDERBUFFER
-      /* obsolete code */
-      GLuint bufferBit = fb->_ColorDrawBit[output][i];
-      /* Set the current read/draw buffer */
-      swrast->CurrentBufferBit = bufferBit;
-      if (swrast->Driver.SetBuffer)
-         (*swrast->Driver.SetBuffer)(ctx, ctx->DrawBuffer, bufferBit);
-#endif
-
       /* make copy of incoming colors */
       MEMCPY( rgbaTmp, span->array->rgba, 4 * span->end * sizeof(GLchan) );
 
@@ -1085,11 +1102,6 @@ multi_write_rgba_span( GLcontext *ctx, struct sw_span *span )
                     span->array->mask);
       }
    }
-
-#if OLD_RENDERBUFFER
-   /* restore default dest buffer */
-   _swrast_use_draw_buffer(ctx);
-#endif
 }
 
 
@@ -1105,11 +1117,11 @@ _swrast_write_rgba_span( GLcontext *ctx, struct sw_span *span)
 {
    const GLuint colorMask = *((GLuint *) ctx->Color.ColorMask);
    SWcontext *swrast = SWRAST_CONTEXT(ctx);
-   const GLuint origInterpMask = span->interpMask;
-   const GLuint origArrayMask = span->arrayMask;
+   const GLbitfield origInterpMask = span->interpMask;
+   const GLbitfield origArrayMask = span->arrayMask;
    const GLboolean deferredTexture = !(ctx->Color.AlphaEnabled ||
                                        ctx->FragmentProgram._Active ||
-                                       ctx->ATIFragmentShader._Enabled);
+                                       ctx->ShaderObjects._FragmentShaderPresent);
 
    ASSERT(span->primitive == GL_POINT  ||  span->primitive == GL_LINE ||
 	  span->primitive == GL_POLYGON  ||  span->primitive == GL_BITMAP);
@@ -1164,6 +1176,10 @@ _swrast_write_rgba_span( GLcontext *ctx, struct sw_span *span)
       interpolate_texcoords(ctx, span);
    }
 
+   if (ctx->ShaderObjects._FragmentShaderPresent) {
+      interpolate_varying(ctx, span);
+   }
+
    /* This is the normal place to compute the resulting fragment color/Z.
     * As an optimization, we try to defer this until after Z/stencil
     * testing in order to try to avoid computing colors that we won't
@@ -1181,6 +1197,14 @@ _swrast_write_rgba_span( GLcontext *ctx, struct sw_span *span)
          interpolate_fog(ctx, span);
 
       /* Compute fragment colors with fragment program or texture lookups */
+#if FEATURE_ARB_fragment_shader
+      if (ctx->ShaderObjects._FragmentShaderPresent) {
+         if (span->interpMask & SPAN_Z)
+            _swrast_span_interpolate_z (ctx, span);
+         _swrast_exec_arbshader (ctx, span);
+      }
+      else
+#endif
       if (ctx->FragmentProgram._Active) {
          /* frag prog may need Z values */
          if (span->interpMask & SPAN_Z)
@@ -1193,9 +1217,11 @@ _swrast_write_rgba_span( GLcontext *ctx, struct sw_span *span)
          _swrast_texture_span( ctx, span );
 
       /* Do the alpha test */
-      if (!_swrast_alpha_test(ctx, span)) {
-         span->arrayMask = origArrayMask;
-	 return;
+      if (ctx->Color.AlphaEnabled) {
+         if (!_swrast_alpha_test(ctx, span)) {
+            span->arrayMask = origArrayMask;
+	    return;
+	 }
       }
    }
 
@@ -1224,17 +1250,13 @@ _swrast_write_rgba_span( GLcontext *ctx, struct sw_span *span)
       }
    }
 
-   /* if we get here, some fragments passed the depth test */
-   if (ctx->Depth.OcclusionTest) {
-      ctx->OcclusionResult = GL_TRUE;
-   }
-
 #if FEATURE_ARB_occlusion_query
-   if (ctx->Occlusion.Active) {
+   if (ctx->Query.CurrentOcclusionObject) {
       /* update count of 'passed' fragments */
+      struct gl_query_object *q = ctx->Query.CurrentOcclusionObject;
       GLuint i;
       for (i = 0; i < span->end; i++)
-         ctx->Occlusion.PassedCounter += span->array->mask[i];
+         q->Result += span->array->mask[i];
    }
 #endif
 
@@ -1262,6 +1284,14 @@ _swrast_write_rgba_span( GLcontext *ctx, struct sw_span *span)
       if (span->interpMask & SPAN_FOG)
          interpolate_fog(ctx, span);
 
+#if FEATURE_ARB_fragment_shader
+      if (ctx->ShaderObjects._FragmentShaderPresent) {
+         if (span->interpMask & SPAN_Z)
+            _swrast_span_interpolate_z (ctx, span);
+         _swrast_exec_arbshader (ctx, span);
+      }
+      else
+#endif
       if (ctx->FragmentProgram._Active)
          _swrast_exec_fragment_program( ctx, span );
       else if (ctx->ATIFragmentShader._Enabled)
@@ -1306,6 +1336,20 @@ _swrast_write_rgba_span( GLcontext *ctx, struct sw_span *span)
       }
    }
 
+   /* Clamp color/alpha values over the range [0.0, 1.0] before storage */
+#if CHAN_TYPE == GL_FLOAT
+   if (ctx->Color.ClampFragmentColor) {
+      GLchan (*rgba)[4] = span->array->rgba;
+      GLuint i;
+      for (i = 0; i < span->end; i++) {
+         rgba[i][RCOMP] = CLAMP(rgba[i][RCOMP], 0.0, CHAN_MAXF);
+         rgba[i][GCOMP] = CLAMP(rgba[i][GCOMP], 0.0, CHAN_MAXF);
+         rgba[i][BCOMP] = CLAMP(rgba[i][BCOMP], 0.0, CHAN_MAXF);
+         rgba[i][ACOMP] = CLAMP(rgba[i][ACOMP], 0.0, CHAN_MAXF);
+      }
+   }
+#endif
+
    if (swrast->_RasterMask & MULTI_DRAW_BIT) {
       /* need to do blend/logicop separately for each color buffer */
       multi_write_rgba_span(ctx, span);
@@ -1399,7 +1443,7 @@ _swrast_read_rgba_span( GLcontext *ctx, struct gl_renderbuffer *rb,
       ASSERT(rb);
       ASSERT(rb->GetRow);
       ASSERT(rb->_BaseFormat == GL_RGB || rb->_BaseFormat == GL_RGBA);
-      ASSERT(rb->DataType == GL_UNSIGNED_BYTE);
+      ASSERT(rb->DataType == CHAN_TYPE);
       rb->GetRow(ctx, rb, length, x + skip, y, rgba + skip);
    }
 }
@@ -1509,3 +1553,70 @@ _swrast_get_values(GLcontext *ctx, struct gl_renderbuffer *rb,
                     (GLubyte *) values + inStart * valueSize);
    }
 }
+
+
+/**
+ * Wrapper for gl_renderbuffer::PutRow() which does clipping.
+ */
+void
+_swrast_put_row(GLcontext *ctx, struct gl_renderbuffer *rb,
+                GLuint count, GLint x, GLint y,
+                const GLvoid *values, GLuint valueSize)
+{
+   GLint skip = 0;
+
+   if (y < 0 || y >= rb->Height)
+      return; /* above or below */
+
+   if (x + (GLint) count <= 0 || x >= rb->Width)
+      return; /* entirely left or right */
+
+   if (x + count > rb->Width) {
+      /* right clip */
+      GLint clip = x + count - rb->Width;
+      count -= clip;
+   }
+
+   if (x < 0) {
+      /* left clip */
+      skip = -x;
+      x = 0;
+      count -= skip;
+   }
+
+   rb->PutRow(ctx, rb, count, x, y,
+              (const GLubyte *) values + skip * valueSize, NULL);
+}
+
+
+/**
+ * Wrapper for gl_renderbuffer::GetRow() which does clipping.
+ */
+void
+_swrast_get_row(GLcontext *ctx, struct gl_renderbuffer *rb,
+                GLuint count, GLint x, GLint y,
+                GLvoid *values, GLuint valueSize)
+{
+   GLint skip = 0;
+
+   if (y < 0 || y >= rb->Height)
+      return; /* above or below */
+
+   if (x + (GLint) count <= 0 || x >= rb->Width)
+      return; /* entirely left or right */
+
+   if (x + count > rb->Width) {
+      /* right clip */
+      GLint clip = x + count - rb->Width;
+      count -= clip;
+   }
+
+   if (x < 0) {
+      /* left clip */
+      skip = -x;
+      x = 0;
+      count -= skip;
+   }
+
+   rb->GetRow(ctx, rb, count, x, y, (GLubyte *) values + skip * valueSize);
+}
